------------------------------
--      Are you local?      --
------------------------------

local roster = nil
local times = {}
local L = AceLibrary("AceLocale-2.2"):new("WitchHunt")
local db

---------------------------------
--      Addon Declaration      --
---------------------------------

WitchHunt = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "Parser-3.0", "Sink-1.0")
local WitchHunt = WitchHunt

local defaults = {
	combatonly = false,
	targetonly = false,
	font = L["Normal"],
	lock = true,
	insertmode = L["Top"],
	height = 160,
	sink10OutputSink = "Witch",
}

local consoleoptions = {
	type = "group",
	name = L["Witch Hunt"],
	desc = L["Simple spell alert."],
	handler = WitchHunt,
	pass = true,
	get = function( key )
			return WitchHunt.db.profile[key]
		end,
	set = function( key, value )
			WitchHunt.db.profile[key] = value
			WitchHunt:UpdateDisplay( key )
		end,
	args = {
		combatonly = {
			name = L["Combat Only"], type = "toggle",
			desc = L["Toggle combat only mode."],
		},
		targetonly = {
			name = L["Target Only"], type = "toggle",
			desc = L["Toggle target only mode."],
		},
		emotes = {
			name = L["Emotes"], type = "toggle",
			desc = L["Toggle showing of emotes."],
		},
		lock = {
			name = L["Lock"], type = "toggle",
			desc = L["Toggle locking of the WitchHunt frame."],
		},
		insertmode = {
			name = L["Insert Mode"], type = "text",
			desc = L["Set the insert mode for the Default frame."],
			validate = { L["Top"], L["Bottom"] },
		},
		font = {
			name = L["Font"], type = "text",
			desc = L["Set the font for the display of messages in the Default frame."],
			validate = {L["Small"], L["Normal"], L["Large"], L["Huge"]},
		},
		height = {
			name = L["Height"], type = "range",
			desc = L["Set the height for the Default frame."],
			min = 40,
			max = 200,
			step = 5,
		},
		test = {
			name = L["Test"], type = "execute",
			desc = L["Test with some dummy WitchHunt messages."],
			func = function()
				WitchHunt:TriggerEvent("WitchHunt_Message", "[Ammo: See it works!]", "Orange")
				WitchHunt:ScheduleEvent("WitchHunt_Message", 1, "*Ammo: extra code*", "Yellow")
				WitchHunt:ScheduleEvent("WitchHunt_Message", 2, "+Ammo: Super duper strength+", "LtBlue")
			end,
		}
	}	
}

local parserEvent = {
	sourceID = false,
	eventType_in = {
		Cast = true,
		Gain = true,
		Fade = true,
	},
	event_in = {
		CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE = true, -- Enemy begins to cast 
		CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF = true, -- Enemy gains buff
		CHAT_MSG_SPELL_CREATURE_VS_CREATURE_BUFF = true, -- Creature begins to cast buff
		CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS = true, -- Creature gains buff
		CHAT_MSG_SPELL_CREATURE_VS_CREATURE_DAMAGE = true, -- Creature begins to cast damage
		CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_BUFFS = true, -- Enemy gains buff
	}
}

------------------------------
--      Initialization      --
------------------------------

function WitchHunt:OnInitialize()
	self:RegisterDB("WitchHuntDB")
	self:RegisterDefaults('profile', defaults)

	self:RegisterSink("Witch", L["Witch Hunt"], nil, "WHFramePrint")

	self:RegisterChatCommand("/wh", "/witchhunt", consoleoptions, "WITCHHUNT")
end

function WitchHunt:OnEnable()
	db = self.db.profile
	if not roster and AceLibrary:HasInstance("Roster-2.1") then
		roster = AceLibrary("Roster-2.1").roster
	end

	-- Register our own events
	self:RegisterEvent("WitchHunt_Message")

	-- Register the WoW events
	self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE")

	-- Register Parser Event
	self:RegisterParserEvent( parserEvent, "OnParserEvent" )
end

function WitchHunt:OnDisable()
	-- no more events to handle
	-- parser:UnregisterAllEvents("WitchHunt")
end

function WitchHunt:UpdateDisplay( key )
	if key == "font" or key == "lock" or key == "insertmode" or key == "height" then
		if not self.msgframe then self:SetupFrames() end
		self:UpdateLock()
		self:SetFont()
		self.msgframe:SetHeight( db.height )
		if db.insertmode == L["Bottom"] then
			self.msgframe:SetInsertMode("BOTTOM")
		else
			self.msgframe:SetInsertMode("TOP")
		end
	end
end

function WitchHunt:SetFont()
	if not self.msgframe then self:SetupFrames() end
	if db.font == L["Huge"] then
		self.msgframe:SetFontObject(GameFontNormalHuge)
	elseif db.font == L["Small"] then
		self.msgframe:SetFontObject(GameFontNormalSmall)
	elseif db.font == L["Large"] then
		self.msgframe:SetFontObject(GameFontNormalLarge)
	else
		self.msgframe:SetFontObject(GameFontNormal)
	end
end

function WitchHunt:SetupFrames()
	self.dragbutton = CreateFrame("Button",nil,UIParent)
	self.dragbutton.owner = self
	self.dragbutton:Hide()
	self.dragbutton:ClearAllPoints()
	self.dragbutton:SetWidth(200)
	self.dragbutton:SetHeight(25)
	
	if db.x and db.y then
		self.dragbutton:SetPoint("TOPLEFT", UIParent, "TOPLEFT", db.x, db.y)
	else 
		self.dragbutton:SetPoint("TOP", UIErrorsFrame, "BOTTOM", 0, 0)
	end	
	self.dragbutton:SetBackdrop({
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = false, tileSize = 16, edgeSize = 16,
		insets = { left = 5, right =5, top = 5, bottom = 5 }
	})
	self.dragbutton:SetBackdropColor(1,0,0,.4)	
	
	self.dragbutton:SetMovable(true)
	self.dragbutton:RegisterForDrag("LeftButton")
	self.dragbutton:SetScript("OnDragStart", function() this.owner.dragbutton:StartMoving() end )
	self.dragbutton:SetScript("OnDragStop",
		function()
			this.owner.dragbutton:StopMovingOrSizing()
			local _,_,_,x,y = this:GetPoint("CENTER")
			this.owner.db.profile.x = x
			this.owner.db.profile.y = y
		end
	)
	
	self.dragtext = self.dragbutton:CreateFontString(nil, "OVERLAY")
	self.dragtext.owner = self
	self.dragtext:SetFontObject(GameFontNormalSmall)
	self.dragtext:ClearAllPoints()
	self.dragtext:SetTextColor(1, 1, 1, 1)
	self.dragtext:SetWidth(200)
	self.dragtext:SetHeight(25)
	self.dragtext:SetPoint("TOPLEFT", self.dragbutton, "TOPLEFT")
	self.dragtext:SetJustifyH("CENTER")
	self.dragtext:SetJustifyV("MIDDLE")
	self.dragtext:SetText(L["DRAG HERE"])
	
	self.msgframe = CreateFrame("MessageFrame")
	self.msgframe.owner = self
	self.msgframe:ClearAllPoints()
	self.msgframe:SetWidth(600)
	self.msgframe:SetHeight(db.height)
	self.msgframe:SetPoint("TOP", self.dragbutton, "TOP", 0, 0)
	if db.insertmode == L["Bottom"] then
		self.msgframe:SetInsertMode("BOTTOM")
	else
		self.msgframe:SetInsertMode("TOP")
	end
	self.msgframe:SetFrameStrata("HIGH")
	self.msgframe:SetToplevel(true)

	self:UpdateLock()
	self:SetFont()

	self.msgframe:Show()
end

function WitchHunt:UpdateLock()
	if not self.msgframe then self:SetupFrames() end
	if db.lock then
		self.dragbutton:SetMovable(false)
		self.dragbutton:RegisterForDrag()
		self.msgframe:SetBackdrop(nil)
		self.msgframe:SetBackdropColor(0,0,0,0)
		self.dragbutton:Hide()
	else
		self.dragbutton:Show()
		self.dragbutton:SetMovable(true)
		self.dragbutton:RegisterForDrag("LeftButton")
		self.msgframe:SetBackdrop({
			bgFile = "Interface/Tooltips/UI-Tooltip-Background",
			edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			tile = false, tileSize = 16, edgeSize = 16,
			insets = { left = 5, right =5, top = 5, bottom = 5 }
		})
		self.msgframe:SetBackdropColor(1,0,0,.4)
	end
end

function WitchHunt:IsFriendly( name )
	if roster then
		if roster[name] then return UnitIsFriend("player", roster[name].unitid) end
		return nil
	end

	if name == UnitName("player") then
		return true
	end
	if name == UnitName("pet") and UnitIsFriend("player", "pet") then
		return true
	end

	local unit
	for i = 1, 4 do
		unit = string.format("%s%d", "party", i)
		if name == UnitName(unit) and UnitIsFriend("player", unit) then
			return true
		end
		unit = string.format("%s%d", "partypet", i)
		if name == UnitName(unit) and UnitIsFriend("player", unit) then
			return true
		end
	end
	local n = GetNumRaidMembers()
	if n > 0 then
		for i = 1, n do
			unit = string.format("%s%d", "raid", i)
			if name == UnitName(unit) and UnitIsFriend("player", unit) then
				return true
			end
			unit = string.format("%s%d", "raidpet", i)
			if name == UnitName(unit) and UnitIsFriend("player", unit) then
				return true
			end
		end
	end
end

do
	local colors = {
		Red    = {r=1, g=0, b=0}, Orange = {r=1, g=0.5, b=0},
		Yellow = {r=1, g=1, b=0}, Green  = {r=0, g=1, b=0},
		LtBlue = {r=0, g=1, b=1}, Blue   = {r=0, g=0, b=1},
		Purple = {r=1, g=0, b=1}, White  = {r=1, g=1, b=1},
		Black  = {r=0, g=0, b=0},
	}
	function WitchHunt:GetColor( color )
		local c = (type(color) == "string" and colors[color]) or color
		if type(c) == "table" then
			if c[1] then return c[1], c[2], c[3]
			elseif c.r and c.g and c.b then return c.r, c.g, c.b end
		end
	end
end

do
	local formats = {
		default = "$c: $e",
		emote = "[$e]",
		spell = "*$c: $e*",
		totem = "$c: $e",
		gain = "$c: +$e+",
		fade = "$c: -$e-",
	}
	-- Burn the witch!
	function WitchHunt:Burn(caster, effect, format, color)
		if not caster or not effect then return end

		-- Are we even on a hunt?
		if db.combatonly and not UnitAffectingCombat("player") then return end

		-- Check if this witch requires burning
		if self:IsFriendly(caster) then return end
		if db.targetonly then
			if not UnitExists("target") then return
			elseif UnitName("target") ~= caster then return end
		end

		-- Can we build this pyre?
		if not formats[format] then format = L["Default"] end

		-- Prepare the pyre!
		local text = formats[format]:gsub("$c", caster):gsub("$e", effect)

		-- Light the Fire!
		local t = GetTime()
		if ( not times[text] ) or ( times[text] and (times[text] + 3) <= t ) then
			times[text] = t
			self:TriggerEvent("WitchHunt_Message", text, color)
		end
	end
end

-- -- -- -- -- -- -- -- -- -- -- -- -- --
-- All events we watch out for         --
-- -- -- -- -- -- -- -- -- -- -- -- -- --

function WitchHunt:WHFramePrint(addon, message, r, g, b)
	if not self.msgframe then self:SetupFrames() end
	self.msgframe:AddMessage(message, r, g, b, 1, UIERRORS_HOLD_TIME)
end

function WitchHunt:WitchHunt_Message(msg, color)
	if not msg then return end
	local r, g, b = self:GetColor(color)
	self:Pour(msg, r, g, b)
end

function WitchHunt:CHAT_MSG_MONSTER_EMOTE(msg, monster)
	if db.emotes then
		self:Burn( monster, msg:format(monster), "emote", "Orange")
	end
end

function WitchHunt:OnParserEvent( info )
	if info.eventType == "Cast" then
		if info.isBegin then
			self:Burn( info.sourceName, info.abilityName, "spell", "Yellow" )
		else
			if info.abilityName:find(L["Totem"]) then
				self:Burn( info.sourceName, info.abilityName, "totem", "Green" )
			else
				self:Burn( info.sourceName, info.abilityName, "spell", "Purple" )
			end
		end
	elseif info.eventType == "Gain" and not info.amount then
		self:Burn(  info.sourceName, info.abilityName, "gain", "LtBlue" )
	elseif info.eventType == "Fade" and not info.amount then
		self:Burn(  info.sourceName, info.abilityName, "fade", "Blue" )
	end
end
